function [deltax,autocorru,kx,Euu]=autocorrandspectrumONE(field,x)
% function used to calculate instantaneous autocorrelation and energy spectrum
% (time-averaging and possibly spatial averaging) in the direction x of a time dependant field component
% field(x,y,z,t) or field(x,y,t) or field(x,t).
% field is a time-depedent 1D spatial vector u(x,t), 2D vector u(x,z,t) (PIV2D), or 3D
% vector u(x,y,z,t) (PIV3D)
% x is an 1D vector containing the real position in the x-direction.
%Fuu is the longitudinal spectrum
%E is the spectrum using E(k)=1/2[k1^3 d/dk1(1/k1 d/dk1(Fuu(k1)))]k=k1
ss=size(field);
temp=size(ss);
ndimspatial=temp(2)-1       % see if u(x,t), u(x,y,t) or even u(x,y,z,t) fields are considered
nbtimesamples=ss(ndimspatial+1);
nx=ss(1);
if (ndimspatial==1)
    fprintf(1,'ensemble averaging is done by time averaging\n');
    u=zeros(nx,1,nbtimesamples);
    umoy=mean(field,2);
    ntotal=1;
    for i=1:nx
        for indt=1:nbtimesamples
            u(i,1,indt)=field(i,indt)-umoy(i);
        end
    end
    ntotal=1;
elseif (ndimspatial==2)
    fprintf(1,'ensemble averaging is done by time and y spatial averaging\n');
    ny=ss(2);
    ntotal=ny;
    umoy=mean(field,3);
    u=zeros(nx,ny,nbtimesamples);
    for i=1:nx
        for j=1:ny
            for indt=1:nbtimesamples
                 u(i,j,indt)=field(i,j,indt)-umoy(i,j);
            end
       end
    end
elseif (ndimspatial==3)
    fprintf(1,'ensemble averaging is done by time and y- and z- spatial averaging\n');
    ny=ss(2);
    ny=ss(3);
    ntotal=ny*nz;
    umoy=mean(field,4);
    u=zeros(nx,ny*nz,nbtimesamples);
    for i=1:nx
        for j=1:ny
        for k=1:nz
            ind=nz*(j-1)+k
            for indt=1:nbtimesamples
               u(i,ind,indt)=field(i,j,k,indt)-umoy(i,j,k);
            end
        end
        end
    end
    umoy=(mean(u,3));
    for ind=1:nbtimesamples
        for i=1:nx
        for j=1:ntotal
            u(i,j,ind)=u(i,j,ind)-umoy(i,j);
        end
        end
    end
end
ind=0;
u2=0.0;
for i=1:nx
    for j=1:ntotal
        for k=1:nbtimesamples
            ind=ind+1;
            u2=u2+u(i,j,k)*u(i,j,k);
        end
    end
end
u2=u2/ind;
urms=sqrt(u2);
% TRAVAIL SUR SPECTRE ENTIER 
kx=2.0*pi*(0:(nx-1))/(x(nx)-x(1));
Euu=zeros(nx,ntotal);
autocorru=zeros(2*nx-1,ntotal);
uploc=zeros(nx,1);
for indj=1:ntotal
for indt=1:nbtimesamples
    uploc(:)=u(:,indj,indt);
    autocorruloc=xcorr(uploc,uploc,'coeff'); % version with signal toolbox available
    %autocorruloc=xcorrI(uploc); %version done here (personnal programming)
    utildeloc=fft(uploc)/(nx);
    if (indt==1)
        Euu(1:(nx),indj)=utildeloc(1:(nx)).*conj(utildeloc(1:(nx)));
        autocorru(1:(2*nx-1),indj)=autocorruloc(1:(2*nx-1));
    else
        Euu(1:(nx),indj)=Euu(1:(nx),indj)+utildeloc(1:(nx)).*conj(utildeloc(1:(nx)));
        autocorru(1:(2*nx-1),indj)=autocorru(1:(2*nx-1),indj)+autocorruloc(1:(2*nx-1));
    end
end
end
Euu=1.0*Euu/(nbtimesamples);
autocorru=autocorru/(nbtimesamples);
temp=mean(Euu,2);
temp2=mean(autocorru,2);
Euu=squeeze(temp)/kx(2);        % DIFFERENCE
autocorru=squeeze(temp2);
% verification of the total integral (Euu should satisfy
% int(-inft->+infty)Euu dkx=urms^2
intEuu=sum(Euu)*kx(2);
fprintf(1,'Int(Euu) from 0 to infty : %u\t, compared to urms^2 %u\t\n',intEuu,urms*urms); 
% determination of the integral Timescale Tint
dx=mean(x(2:nx)-x(1:(nx-1))); 
deltaxcorr=(0:(nx-1))*dx;
deltax=deltaxcorr;
ind=nx;
while (autocorru(ind)>0.01)
   ind=ind+1; 
end
ind1p100=ind;
Lcut1p100=deltaxcorr((ind-nx)+1);
Lint1p100=sum(autocorru(nx:ind))*dx;
ind=nx;
while (autocorru(ind)>0.1)
   ind=ind+1; 
end
Lcut=deltaxcorr((ind-nx)+1);
Lint=sum(autocorru(nx:ind))*dx;
   fprintf(1,'Integrale length scale Lint : %u\t (with 0.1 cut) %u\t (with 0.01 cut)\n', Lint,Lint1p100); 
if (deltaxcorr(nx)<(8.0*Lint))
   fprintf(1,'Beware !!! sample length seems to be smaller than 8 Lint !!! \n'); 
end

% estimation of taylor microscale near the top (the dx=0 value is discarded
% since contaminated by the noise rms urmsnoise^2/urmstotal^2
ltaylor=2;  % number of points of the autocorrelation curve chosen for the fitting
npoly=2;    % polynome degree for the fitting
xx=zeros(2*ltaylor,1);
yy=zeros(2*ltaylor,1);
for i=1:ltaylor
   xx(ltaylor+i)=dx*i;
   xx(ltaylor+1-i)=-dx*i;
   yy(ltaylor+i)=autocorru(nx+i);   
   yy(ltaylor+1-i)=autocorru(nx-i);   
end
%yy=autocorru(nx-ltaylor:nx+ltaylor);
%xx=dx*[(nx-ltaylor):(nx+ltaylor)];
[p]=polyfit(xx,yy,npoly);
derivsec=2.0*p(npoly-1);
Ltaylor=sqrt(-2.0/derivsec);
derivsec=-2.0*(autocorru(nx)-autocorru(nx+1))/(dx*dx);
Ltaylordiff=sqrt(-2.0/derivsec);
fprintf(1,'Taylor length scale Ltaylor : %u\t (with polynomial fit without central peak) %u\t (with finite diff)\n', Ltaylor, Ltaylordiff);
fprintf(1,'Corrected autocorrelation at dx=0 : %u\n',p(npoly+1));
uncorr=p(npoly+1); % here is the estimated real value at dx=0 after noise correction

% plot of the autocorrelation
figure;
plot(deltaxcorr,autocorru((nx):(2*nx-1)),'go');
hold on
maxL=4.0*Ltaylor;
mincorr=-0.1;
maxcorr=1.1;
plot([0 maxL],[0 0],'k');
plot([Lcut Lcut],[mincorr 0.1]);
plot([Lcut1p100 Lcut1p100],[mincorr 0.01]);
plot([Lint Lint],[0 maxcorr],'r');
plot([Lint1p100 Lint1p100],[0 maxcorr],'r');
tab=[0:100]/100*Ltaylor;
ytab=uncorr-tab.*tab/(Ltaylor*Ltaylor);
plot(tab,ytab,'r');
tab=[0:100]/100*Ltaylordiff;
ytab=1.0-tab.*tab/(Ltaylordiff*Ltaylordiff);
plot(tab,ytab,'b');
axis([0 maxL mincorr maxcorr]);

% plot of the adimensional spectrum, using the taylor lengthscale
% Euu semble avoir la bonne dimension
Euuadim=(2.0*pi)^2*Euu/(urms*Lint);
kxadim=kx*Lint;
kxtayloradim=2.0*pi/Ltaylor*Lint;
kxtaylordiffadim=2.0*pi/Ltaylordiff*Lint;
figure;
plot(log10(kxadim),log10(Euuadim));
hold on
minikxadim=log10(kxadim(1));
maxikxadim=log10(kxadim(nx/2-1));
Emin=log10(Euuadim(nx/2-1));
Emax=log10(Euuadim(1));
%minikxadim=-2.5
%maxikxadim=0.0
%Emin=-2.0;
%Emax=1.5
axis([minikxadim maxikxadim Emin Emax]);
%dec=-1.;
%plot([minifreqadim maxifreqadim],[log10(Euuadim(tailletps/2-1))+5.0/3.0*(maxifreqadim-minifreqadim)+dec log10(Euuadim(tailletps/2-1))+dec]);
hold on
plot([log10(2.0*pi) log10(2.0*pi)],[Emin Emax],'g');
plot([log10(kxtayloradim) log10(kxtayloradim)],[Emin Emax],'r');
plot([log10(kxtaylordiffadim) log10(kxtaylordiffadim)],[Emin Emax],'r');
% polynomial fitting
x=log10(kxadim);
y=log10(Euuadim);
[p]=polyfit(x',y,5);
f = polyval(p,x);
%plot(x,f,'g');
ecartlog10Euu=f'-y;
rmsspectre=sqrt(mean(ecartlog10Euu.*ecartlog10Euu));
%plot(x,f,'g');
y=-5.0/3.0*log10(kxadim);
posdep=max(log10(Euuadim)-y');
plot(x,y+posdep,'g'); 
%plot(x,y+posdep-4.0*rmsspectre,'r');
end

function [data]=xcorrI(u)
N=size(u);
data=zeros(2*N-1,1);
for i=N:(2*N-1)
    dec=i-N;
    produit=0.0;
    for j=1:N
        xloc=u(j);
        yloc=0.0;
        if ((j+dec)<=N)
            yloc=u(j+dec)
        end
        produit=produit+xloc*yloc;
    end
    data(i)=produit;
end
data
for i=1:(N-1)
    ind=-(i-N);
    data(i)=data(N+ind);
end
Ref=data(N);
for i=1:(2*N-1)
    data(i)=data(i)/Ref;
end
end

    