function [deltatcorr,autocorrutr,freqs,Euu,urms,Ttaylor,Tint,epsilon,epsilonturb]=autocorrandspectrum(tps,u,Nblock)
% function used to calculate autocorrelation and energy spectrum
% (time-averaging) of one field component u using 2^Nblock sub samples
% It's better to truncate the time series into a power 2 sample size, to
% get urms^2=int(Euu(omega)domega for the larger value of Nblock available
ss=size(tps);
nbsamples=ss(1);
nbseries=nbsamples/(2^Nblock);
tailletps=2^Nblock
umoy=mean(u);
up=u-umoy;
u2=up.*up;
urms=sqrt(mean(u2));
for iseries=1:(nbsamples-tailletps+1)
    inddeb=iseries;
    indfin=iseries+tailletps-1;
    %umoy=mean(u(inddeb:indfin));    % could influence the calculus
    up=u(inddeb:indfin)-umoy;
    u2loc=up.*up;
%    urms=sqrt(mean(u2loc));
    autocorruloc=xcorr(up,up,'coeff');
    utildeloc=fft(up)/(2^Nblock);
    if (iseries==1)
        %u2tildemoy=u2tildeloc;
        Euu=utildeloc(2:(tailletps/2)).*conj(utildeloc(2:(tailletps/2)));
        Euutot=utildeloc.*conj(utildeloc);
        autocorru=autocorruloc;
    else
%        u2tildemoy=u2tildemoy+u2tildeloc;
        Euu=Euu+utildeloc(2:(tailletps/2)).*conj(utildeloc(2:(tailletps/2)));
        Euutot=Euutot+utildeloc.*conj(utildeloc);
        autocorru=autocorru+autocorruloc;
    end    
end
%        u2tildemoy=u2tildemoy/nbseries;
dt=mean(tps(2:nbsamples)-tps(1:(nbsamples-1)));
%dtrms=sqrt(mean((tps(2:nbsamples)-tps(1:(nbsamples-1))-dt).*(tps(2:nbsamples)-tps(1:(nbsamples-1))-dt),1);
dtrms=sqrt(mean((tps(2:nbsamples)-tps(1:(nbsamples-1))-dt).*(tps(2:nbsamples)-tps(1:(nbsamples-1))-dt)))
fprintf(1,'dt=%u\t with relative variation of time intervals dtrms/dt : %u\n',dt,dtrms/dt); 
if (dtrms/dt>0.01)
   fprintf(1,'time samples are not equally spaced, it is not the right postprocessing !!! \n'); 
   stop
end
freq=2.0*pi*(1:(tailletps/2-1))/(dt*(tailletps-1));
Euu=2.0*Euu/(nbsamples-tailletps+1)/freq(1);
Euutot=Euutot/(nbsamples-tailletps+1)/freq(1);
autocorru=autocorru/(nbsamples-tailletps+1);
% verification of the total integral (Euu should satisfy
% int(-infty->+infty)Euu domega=urms^2
intEuu=sum(Euu)*(freq(2)-freq(1));
fprintf(1,'sqrt(Int(Euu)) from 1 to infty : %u\t, compared to urms %u\t\n',sqrt(intEuu),urms); 
urmsturb=sqrt(intEuu);
intEuu=sum(Euutot)*(freq(2)-freq(1));
fprintf(1,'sqrt(Int(Euu)) from 0 to infty : %u\t, compared to urms %u\t\n',sqrt(intEuu),urms); 
% determination of the integral Timescale Tint
deltatcorr=(0:(tailletps-1))*dt;
ind=tailletps;
while (autocorru(ind)>0.01)
   ind=ind+1; 
end
ind1p100=ind;
taucut1p100=deltatcorr((ind-tailletps)+1);
Tint1p100=sum(autocorru(tailletps:ind))*dt;
ind=tailletps;
while (autocorru(ind)>0.1)
   ind=ind+1; 
end
taucut=deltatcorr((ind-tailletps)+1);
Tint=sum(autocorru(tailletps:ind))*dt;
fprintf(1,'Integrale time scale Tint : %u\t (with 0.1 cut) %u\t (with 0.01 cut)\n', Tint,Tint1p100); 
if (deltatcorr(tailletps)<(8.0*Tint))
   fprintf(1,'Beware !!! time sample duration seems to be smaller than 8 Tint !!! \n'); 
end

% estimation of taylor microscale near the top (the dx=0 value is discarded
% since contaminated by the noise rms urmsnoise^2/urmstotal^2
ltaylor=2;  % number of points of the autocorrelation curve chosen for the fitting
npoly=2;    % polynome degree for the fitting
%xx=zeros(2*ltaylor,1);
%yy=zeros(2*ltaylor,1);
xx=zeros(2*ltaylor+1,1);
yy=zeros(2*ltaylor+1,1);
for i=1:ltaylor
%   xx(ltaylor+i)=dt*i;
%   xx(ltaylor+1-i)=-dt*i;
%   yy(ltaylor+i)=autocorru(tailletps+i);   
%   yy(ltaylor+1-i)=autocorru(tailletps-i);   
   xx(ltaylor+1+i)=dt*i;
   xx(ltaylor+1-i)=-dt*i;
   yy(ltaylor+1+i)=autocorru(tailletps+i);   
   yy(ltaylor+1-i)=autocorru(tailletps-i);   
end
xx(ltaylor+1)=0.0;
yy(ltaylor+1)=1.0;
[p]=polyfit(xx,yy,npoly);
derivsec=2.0*p(npoly-1);
Ttaylor=sqrt(-2.0/derivsec);
derivsec=-2.0*(autocorru(tailletps)-autocorru(tailletps+1))/(dt*dt);
Ttaylordiff=sqrt(-2.0/derivsec);
fprintf(1,'Taylor time scale Ttaylor : %u\t (with polynomial fit without central peak) %u\t (with finite diff)\n', Ttaylor, Ttaylordiff);
fprintf(1,'Corrected autocorrelation at dtau=0 : %u\n',p(npoly+1));
uncorr=p(npoly+1); % here is the estimated real value at dx=0 after noise correction

% plot of the autocorrelation
figure;
plot(deltatcorr,autocorru((tailletps):(2*tailletps-1)),'go');
autocorrutr=autocorru((tailletps):(2*tailletps-1));
hold on
maxtau=4.0*Tint;
mincorr=-0.1;
maxcorr=1.1;
plot([0 maxtau],[0 0],'k');
plot([taucut taucut],[0 0.1]);
plot([taucut1p100 taucut1p100],[0 0.01]);
plot([Tint Tint],[0 maxcorr],'r');
plot([Tint1p100 Tint1p100],[0 maxcorr],'r');
tab=[0:100]/100*Ttaylor;
ytab=uncorr-tab.*tab/(Ttaylor*Ttaylor);
plot(tab,ytab,'b');
tab=[0:100]/100*Ttaylordiff;
ytab=1.0-tab.*tab/(Ttaylordiff*Ttaylordiff);
plot(tab,ytab,'b');
axis([0 maxtau mincorr maxcorr]);
xlabel('elapsed time \Delta t (s)')
%ylabel('log10(E_{uu}(\omega)* \frac{(2 \pi)^2}{u_{rms} L_{int}})')
ylabel('R_{uu}(\Delta t)/ u_{rms}^2')
title('Scaled autocorrelation fonction R_{uu}(\Delta t)')

Ttaylor=Ttaylordiff;
fprintf(1,'Taylor time scale adopted : with finite diff\n');
% plot of the adimensional spectrum
Lint=urms*Tint;
%freqadim=freq*Lint/urms;
%freqtayloradim=2.0*pi/Ttaylor*Lint/urms;
%freqintadim=2.0*pi/Tint*Lint/urms;
freqadim=freq*Ttaylor;
freqtayloradim=2.0*pi/Ttaylor*Ttaylor;
freqintadim=2.0*pi/Tint*Ttaylor;
Euuadim=(2.0*pi)^2*Euu/(urms*Lint);
figure;
plot(log10(freqadim),log10(Euuadim));
hold on
minifreqadim=log10(freqadim(1));
maxifreqadim=log10(freqadim(tailletps/2-1));
Emin=log10(Euuadim(tailletps/2-1));
Emax=log10(Euuadim(1));
%minifreqadim=-2.5
%maxifreqadim=0.0
%Emin=-2.0;
%Emax=1.5
axis([minifreqadim maxifreqadim Emin Emax]);
%dec=-1.;
%plot([minifreqadim maxifreqadim],[log10(Euuadim(tailletps/2-1))+5.0/3.0*(maxifreqadim-minifreqadim)+dec log10(Euuadim(tailletps/2-1))+dec]);
hold on
plot([log10(freqtayloradim) log10(freqtayloradim)],[Emin Emax],'b'); % echelle de taylor
plot([log10(freqintadim) log10(freqintadim)],[Emin Emax],'r');       % echelle integrale
% polynomial fitting
x=log10(freqadim);
y=log10(Euuadim);
[p]=polyfit(x',y,5);
f = polyval(p,x);
%plot(x,f,'g');
ecartlog10Euu=f'-y;
rmsspectre=sqrt(mean(ecartlog10Euu.*ecartlog10Euu));
y=-5.0/3.0*log10(freqadim);
posdep=max(log10(Euuadim)-y');
plot(x,y+posdep,'k--'); 
plot(x,y,'k-'); 
%xlabel('log10(\omega*T_{taylor})= log10(\frac{2\pi}{T_{total}}*T_{Taylor})')
xlabel('\omega*T_{taylor}')
%ylabel('log10(E_{uu}(\omega)* \frac{(2 \pi)^2}{u_{rms} L_{int}})')
ylabel('E_{uu}(\omega)*(2 \pi)^2 / (u_{rms} L_{int})')
title('Temporal spectrum E_{uu}(\omega) in nondimensional representation')
%plot(x,y+posdep-4.0*rmsspectre,'r');
% ESTIMATION EPSILON VIA AL HOMOUD where Phi(omega)=alpha epsilon^2/3*omega^-5/3
% with alpha = around Unity
freqs=freq';
fitinf=2.0*pi/Tint;
[toto,indinf]=max((freq-fitinf)>0);
fitsup=2.0*pi/Ttaylor;
[toto,indsup]=max((freq-fitsup)>0);
dec=log10(Euu)+5.0/3.0*log10(freqs);
moyenne=squeeze(mean(dec(indinf:indsup)));
epsilon=(exp(moyenne*log(10)))^1.5/urms;
fprintf(1,'Epsilon estimated from Al Hamoud formula with alpha=1 and total urms :%u\n',epsilon);
epsilonturb=(exp(moyenne*log(10)))^1.5/urmsturb;
fprintf(1,'Epsilon estimated from Al Hamoud formula with alpha=1 and purely turbulent urms :%u\n',epsilonturb);
%figure;
%plot(log10(freqs),log10(Euu),'k');
%hold on
%plot([log10(fitinf) log10(fitsup)],[log10((epsilon*urms)^(2.0/3.0)*fitinf^(-5.0/3.0)) log10((epsilon*urms)^(2.0/3.0)*fitsup^(-5.0/3.0))],'r');



    
    




